# -*- coding: utf-8 -*-

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from ckeditor.fields import RichTextField
from cms.models.pluginmodel import CMSPlugin
from easy_thumbnails.files import get_thumbnailer
from filer.fields.file import FilerFileField
from filer.fields.image import FilerImageField
from image_cropping import ImageRatioField
from taggit.models import Tag
from thx.core.models import UserModel, DateModel, StatusModel, CleanModel, OrderedModel
from thx.filer_image_cropping.fields import CroppableFilerImageField


class SectionSettings(CMSPlugin, UserModel, DateModel, StatusModel, CleanModel, OrderedModel):
    title = models.CharField(
        max_length=255,
        verbose_name=_('title')
    )
    slug = models.SlugField(
        max_length=255,
        verbose_name=_('slug')
    )
    template_name = models.CharField(
        max_length=255,
        choices=settings.CMS_BOX_SECTION_TEMPLATE_CHOICES,
        default=settings.CMS_BOX_SECTION_TEMPLATE_DEFAULT,
        verbose_name=_('template')
    )
    classes = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('classes')
    )
    subtitle = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('subtitle')
    )
    description = RichTextField(
        blank=True, null=True,
        verbose_name=_('description')
    )
    truncate = models.IntegerField(
        blank=True, null=True,
        verbose_name=_('truncate')
    )
    image = CroppableFilerImageField(
        blank=True, null=True,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_image',
        verbose_name=_('image')
    )
    use_cropping = models.BooleanField(
        default=False,
        verbose_name=_('use cropping')
    )
    image_cropping = ImageRatioField(
        'image',
        settings.CMS_BOX_SECTION_IMAGE_CROPPING,
        verbose_name=_('image cropping')
    )
    mobile_image = CroppableFilerImageField(
        blank=True, null=True,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_mobile_image',
        verbose_name=_('mobile image')
    )
    use_mobile_cropping = models.BooleanField(
        default=False,
        verbose_name=_('use mobile cropping')
    )
    mobile_image_cropping = ImageRatioField(
        'mobile_image',
        settings.CMS_BOX_SECTION_MOBILE_IMAGE_CROPPING,
        verbose_name=_('mobile image cropping')
    )
    caption = models.CharField(
        max_length=512,
        blank=True, null=True,
        verbose_name=_('caption')
    )
    icon = FilerFileField(
        blank=True, null=True,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_icon',
        verbose_name=_('icon')
    )
    cta_label = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('call to action label')
    )
    cta_link = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('call to action link')
    )
    action_on_file_label = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('action on file label')
    )
    file = FilerFileField(
        blank=True, null=True,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_file',
        verbose_name=_('file')
    )
    order_by = models.CharField(
        max_length=255,
        default=settings.CMS_BOX_SECTION_ORDER_BY_DEFAULT,
        verbose_name=_('order by')
    )

    class Meta:
        verbose_name = _('section')
        verbose_name_plural = _('sections')
        ordering = ('ordering', 'title')
        permissions = (
            ("list_section", "Can list section"),
            ("detail_section", "Can detail section"),
            ("disable_section", "Can disable section"),
        )

    def __str__(self):
        return '[{}] {}'.format('ON' if self.status else 'OFF', self.title)
    
    def clean(self):
        order_by_fields = settings.CMS_BOX_SECTION_ORDER_BY_FIELDS.get(
            self.template_name, 
            settings.CMS_BOX_SECTION_ORDER_BY_DEFAULT_FIELDS
        )

        ordering_fields = self.order_by.lower().replace(" ", "").split(",")

        for ordering_field in ordering_fields:
            if ordering_field not in order_by_fields:
                raise ValidationError({
                    'order_by': ValidationError(_('Insert valid fields for order_by')),
                })

    def get_cropped_image(self):
        if self.image:
            return get_thumbnailer(
                self.image
            ).get_thumbnail({
                'size': (self.image.width, self.image.height),
                'box': self.image_cropping,
                'crop': True,
                'detail': True,
            }).url
        else:
            return False

    def get_cropped_mobile_image(self):
        if self.mobile_image:
            return get_thumbnailer(
                self.mobile_image
            ).get_thumbnail({
                'size': (self.mobile_image.width, self.mobile_image.height),
                'box': self.mobile_image_cropping,
                'crop': True,
                'detail': True,
            }).url
        else:
            return False


class BoxAbstractSettings(CMSPlugin, UserModel, DateModel, StatusModel, CleanModel, OrderedModel):
    title = models.CharField(
        max_length=255,
        verbose_name=_('title')
    )
    slug = models.SlugField(
        max_length=255,
        verbose_name=_('slug')
    )
    classes = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('classes')
    )
    subtitle = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('subtitle')
    )
    description = RichTextField(
        blank=True, null=True,
        verbose_name=_('description')
    )
    truncate = models.IntegerField(
        blank=True, null=True,
        verbose_name=_('truncate')
    )
    image = CroppableFilerImageField(
        blank=True, null=True,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_image',
        verbose_name=_('image')
    )
    use_cropping = models.BooleanField(
        default=False,
        verbose_name=_('use cropping')
    )
    image_cropping = ImageRatioField(
        'image',
        settings.CMS_BOX_BOX_IMAGE_CROPPING,
        verbose_name=_('image cropping')
    )
    mobile_image = CroppableFilerImageField(
        blank=True, null=True,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_mobile_image',
        verbose_name=_('mobile image')
    )
    use_mobile_cropping = models.BooleanField(
        default=False,
        verbose_name=_('use mobile cropping')
    )
    mobile_image_cropping = ImageRatioField(
        'mobile_image',
        settings.CMS_BOX_BOX_MOBILE_IMAGE_CROPPING,
        verbose_name=_('mobile image cropping')
    )
    caption = models.CharField(
        max_length=512,
        blank=True, null=True,
        verbose_name=_('caption')
    )
    icon = FilerFileField(
        blank=True, null=True,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_icon',
        verbose_name=_('icon')
    )
    first_cta_label = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('first call to action label')
    )
    first_cta_link = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('first call to action link')
    )
    second_cta_label = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('second call to action label')
    )
    second_cta_link = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('second call to action link')
    )
    third_cta_label = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('third call to action label')
    )
    third_cta_link = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('third call to action link')
    )
    external_cta_label = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('external call to action label')
    )
    external_cta_link = models.URLField(
        blank=True, null=True,
        verbose_name=_('external call to action link')
    )
    action_on_file_label = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('action on file label')
    )
    file = FilerFileField(
        blank=True, null=True,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_file',
        verbose_name=_('file')
    )

    class Meta:
        abstract = True

    def get_cropped_image(self):
        if self.image:
            return get_thumbnailer(
                self.image
            ).get_thumbnail({
                'size': (self.image.width, self.image.height),
                'box': self.image_cropping,
                'crop': True,
                'detail': True,
            }).url
        else:
            return False
        
    def get_cropped_mobile_image(self):
        if self.mobile_image:
            return get_thumbnailer(
                self.mobile_image
            ).get_thumbnail({
                'size': (self.mobile_image.width, self.mobile_image.height),
                'box': self.mobile_image_cropping,
                'crop': True,
                'detail': True,
            }).url
        else:
            return False


class BoxSettings(BoxAbstractSettings):
    template_name = models.CharField(
        max_length=255,
        choices=settings.CMS_BOX_BOX_TEMPLATE_CHOICES,
        default=settings.CMS_BOX_BOX_TEMPLATE_DEFAULT,
        verbose_name=_('template')
    )

    class Meta:
        verbose_name = _('box')
        verbose_name_plural = _('boxes')
        ordering = ('ordering', 'title')
        permissions = (
            ("list_box", "Can list box"),
            ("detail_box", "Can detail box"),
            ("disable_box", "Can disable box"),
        )

    def __str__(self):
        return '[{}] {}'.format('ON' if self.status else 'OFF', self.title)


class ModelBoxSettings(BoxAbstractSettings):
    template_name = models.CharField(
        max_length=255,
        choices=settings.CMS_BOX_MODEL_BOX_TEMPLATE_CHOICES,
        default=settings.CMS_BOX_MODEL_BOX_TEMPLATE_DEFAULT,
        verbose_name=_('template')
    )
    filter = models.CharField(
        max_length=255,
        blank=True,
        choices=settings.CMS_BOX_MODEL_BOX_FILTER_CHOICES,
        verbose_name=_('filter')
    )
    order_by = models.CharField(
        max_length=255,
        default=settings.CMS_BOX_MODEL_BOX_ORDER_BY_DEFAULT,
        verbose_name=_('order by')
    )
    objects_number = models.IntegerField(
        default=settings.CMS_BOX_MODEL_BOX_OBJECTS_NUMBER_DEFAULT,
        verbose_name=_('objects number')
    )
    typology = models.CharField(
        max_length=3,
        blank=True,
        choices=settings.DOCUMENT_MEDIA_TYPOLOGY_CHOICES,
        verbose_name=_('typology')
    )
    tags = models.ManyToManyField(
        Tag,
        blank=True,
        related_name='model_box_settings',
        verbose_name=_('tags')
    )

    class Meta:
        verbose_name = _('model box')
        verbose_name_plural = _('model boxes')
        ordering = ('ordering', 'title')
        permissions = (
            ("list_modelbox", "Can list model box"),
            ("detail_modelbox", "Can detail model box"),
            ("disable_modelbox", "Can disable model box"),
        )

    def __str__(self):
        return '[{}] {}'.format('ON' if self.status else 'OFF', self.title)
    
    def clean(self):
        order_by_fields = settings.CMS_BOX_MODELBOX_ORDER_BY_DEFAULT_FIELDS
        if self.filter:
            order_by_fields = settings.CMS_BOX_MODELBOX_ORDER_BY_FIELDS.get(self.filter, order_by_fields)

        ordering_fields = self.order_by.lower().replace(" ", "").split(",")

        for ordering_field in ordering_fields:
            if ordering_field not in order_by_fields:
                raise ValidationError({
                    'order_by': ValidationError(_('Insert valid fields for order_by')),
                })
    
    def get_objects(self):
        order_by = self.order_by.lower().replace(" ", "").split(",")
        limit = self.objects_number

        if self.filter:
            klass = import_string(self.filter)
            objects = klass.objects.filter(status=1)
        else:
            return None

        tags = []

        for tag in self.tags.all():
            tags.append(tag.name)

        if self.typology:
            objects = objects.filter(typology=self.typology).distinct()

        if tags:
            objects = objects.filter(tags__name__in=tags).distinct()

        if order_by:
            objects = objects.order_by(*order_by)

        if limit:
            objects = objects[:limit]

        return objects

    def copy_relations(self, oldinstance):
        self.tags.all().delete()
        self.tags.set(oldinstance.tags.all())


class FilterBoxSettings(BoxAbstractSettings):
    template_name = models.CharField(
        max_length=255,
        choices=settings.CMS_BOX_FILTER_BOX_TEMPLATE_CHOICES,
        default=settings.CMS_BOX_FILTER_BOX_TEMPLATE_DEFAULT,
        verbose_name=_('template')
    )

    class Meta:
        verbose_name = _('filter box')
        verbose_name_plural = _('filter boxes')
        ordering = ('ordering', 'title')
        permissions = (
            ("list_filterbox", "Can list filter box"),
            ("detail_filterbox", "Can detail filter box"),
            ("disable_filterbox", "Can disable filter box"),
        )

    def __str__(self):
        return '[{}] {}'.format('ON' if self.status else 'OFF', self.title)

    def copy_relations(self, oldinstance):
        self.label_tag_settings.all().delete()
        for label_tag in oldinstance.label_tag_settings.all():
            label_tag.pk = None
            label_tag.plugin = self
            label_tag.save()


class LabelTagSettings(UserModel, DateModel, StatusModel, OrderedModel):
    tag = models.ForeignKey(
        Tag,
        on_delete=models.CASCADE,
        verbose_name=_('tag')
    )
    label = models.CharField(
        max_length=255,
        verbose_name=_('label')
    )
    plugin = models.ForeignKey(
        FilterBoxSettings,
        on_delete=models.CASCADE,
        related_name="label_tag_settings",
        verbose_name=_("plugin")
    )
